// (C) 2003 Ulrich Hecht <uli@emulinks.de>
// if2a patch by D. Gauchard <deyv@free.fr>
// Licensed under the terms of the GNU Public License version 2

#include <stdio.h>
#include <usb.h>
#include <sys/wait.h>
#include <fcntl.h>
#include "f2a.h"

#include "f2afirm.hex-src.h"

#define EZ "/proc/ezusb/dev0"

struct usb_bus *busses;
struct usb_bus *bus;
struct usb_device *dev;
struct usb_device *f2a=NULL;
usb_dev_handle *f2ahandle;

int usb_connect()
{
	int result;
	int firmware_loaded=0;

	usb_init();
	usb_find_busses();

	find_f2a:
	usb_find_devices();
	busses=usb_busses;

	for (bus=busses; bus; bus=bus->next) {
		for (dev=bus->devices; dev; dev=dev->next) {
			if ((dev->descriptor.idVendor == 0x547) &&
			    (dev->descriptor.idProduct == 0x1002))
				f2a=dev;
			if ((dev->descriptor.idVendor == 0x547) &&
			    (dev->descriptor.idProduct == 0x2131) &&
			     !firmware_loaded)
			{
				int f, w, tot;
				
				if (verbose)
					printf("Uploading ezusb firmware in "EZ"\n");
				
				if ((f = open(EZ, O_WRONLY)) == -1)
				{
					perror(EZ);
					return -1;
				}
				for (tot = 0; tot < sizeof(f2afirm_hex); tot += w)
				{
					w = write(f, f2afirm_hex + tot, sizeof(f2afirm_hex) - tot);
					if (w == -1)
					{
						perror("write on "EZ);
						return -1;
					}
					if (verbose > 1)
					{
						printf(EZ": wrote %i bytes [%i->%i / %i]\r", w, tot, tot+w, sizeof(f2afirm_hex));
						if (verbose > 2) printf("\n");
					}
				}
				close(f);
			
				firmware_loaded=1;
				sleep(2);	// give the EZUSB some time to renumerate
				goto find_f2a;
			}
		}
	}

	if(f2a==NULL) {
		fprintf(stderr, "error: couldn't find f2a attached to usb\n");
		return -1;
	}

	f2ahandle=usb_open(f2a);

	result=usb_claim_interface(f2ahandle, 0x4);
	if(result==-1) {
		fprintf(stderr, "usb_claim_interface: %s\n", usb_strerror());
		return -1;
	}

	result=usb_claim_interface(f2ahandle, 0x83);
	if(result==-1) {
		fprintf(stderr, "usb_claim_interface: %s\n", usb_strerror());
		return -1;
	}
	return 0;
}

void usb_disconnect () {
	usb_release_interface(f2ahandle, 0);
	usb_close(f2ahandle);
}

int usb_read (char *buffer, int buffer_size) {
	int result;

	result=usb_bulk_read(f2ahandle, 0x83, buffer, buffer_size, USB_TIMEOUT);
	if(result==-1) {
		fprintf(stderr, "usb_bulk_read: %s\n", usb_strerror());
		return -1;
	}
	return result;
}

int usb_write(char *buffer, int buffer_size) {
	int result;

	result=usb_bulk_write(f2ahandle, 0x4, buffer, buffer_size, USB_TIMEOUT);
	if(result==-1) {
		fprintf(stderr, "usb_bulk_write: %s\n", usb_strerror());
		return -1;
	}
	return result;
}


