// (C) 2003 Ulrich Hecht <uli@emulinks.de>
// if2a patch by D. Gauchard <deyv@free.fr>
// Licensed under the terms of the GNU Public License version 2

#include "f2a.h"
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>

int sim = 0;

int f2a_writemem(unsigned char* rom, int base, int offset, int size, int blocksize)
{
	int i;
	sendmsg sm;

	if (verbose)
		printf("Burning... (base=0x%x offset=0x%x size=0x%x)\n", base, offset, size);

	// flash the ROM

	// initialize command buffer
	memset(&sm, 0, sizeof(sm));
	sm.command=CMD_WRITEDATA;
	sm.magic=MAGIC_NUMBER;
	//sm.unknown=0xa;		// no idea what this is...
	// From sniffer under windows : 6: GameSRAM a: GameROM
	sm.unknown=(base >= 0xe000000)? 0x06: 0x0a;
	
	sm.size = size;
	sm.address = base + offset;
	sm.sizekb = size >> 10;

	if (!sim)
		if (sendf2a(sm) == -1)
		{
			fprintf(stderr, "Error sending command\n");
			return -1;
		}

	for(i = 0; i < size; i += blocksize)
	{
		if (!sim)
		{
			if (verbose > 1)
				printf("writing 0x%x bytes at base 0x%x offset 0x%x\n", blocksize, base, offset + i);
			if (usb_write(&rom[i], blocksize) == -1)
			{
 				fprintf(stderr, "Error sending data\n");
				return -1;
			}
		}
		if (verbose)
		{
			printf("wrote chunk %i/%i (%i)\r", (i + blocksize) >> 17, size >> 17, (i + blocksize + offset) >> 17);
			fflush(stdout);
		}
	}
	
	if(verbose)
		printf("\nDone\n");

	return 0;
}

int f2a_writememtofiles (int numfiles, char** files, int address)
{
	sendmsg sm;

	int i,j;
	int romf;
	unsigned char rom[1024];
	struct stat st;
	
	memset(&sm, 0, sizeof(sm));

	// initialize command buffer
	sm.command=CMD_WRITEDATA;
	sm.magic=MAGIC_NUMBER;
	sm.unknown=0xa;		// no idea what this is...
	
	for(j=0; j<numfiles; j++)
	{
		if(verbose) printf("sending file %d: %s\n", j, files[j]);
		
		if(stat(files[j],&st)==-1)
		{
			perror("failed to stat file");
			return -1;
		}

		// round up to 32k, FIXME: This has to be 128k for Turbo carts
		if(st.st_size & (32768-1)) st.st_size+=32768;
		st.st_size&=~(32768-1);

		romf=open(files[j],O_RDONLY);

		// flash the ROM
		sm.size=st.st_size;
		sm.address=address;
		sm.sizekb=st.st_size/1024;

		if(sendf2a(sm)==-1) return -1;

		for(i=0;i<st.st_size;i+=1024)
		{
			//printf("writing chunk %d\n", i);
			if(read(romf, rom, 1024)==-1)
			{
				perror("failed to read from file");
				return -1;
			}
			if(usb_write(rom,1024)==-1) return -1;
		}

		close(romf);
		address+=st.st_size;
	}

	return 0;
}
